local config = require('mer.characterBackgrounds.config')

local getData = function()
    local data = tes3.player.data.merBackgrounds
    return data
end

local fireEffects = {
    [tes3.effect.fireShield] = true,
    [tes3.effect.fireDamage] = true,
    [tes3.effect.weaknesstoFire] = true,
    [tes3.effect.resistFire] = true,
}
local frostEffects = {
    [tes3.effect.frostShield] = true,
    [tes3.effect.frostDamage] = true,
    [tes3.effect.weaknesstoFrost] = true,
    [tes3.effect.resistFrost] = true,
}

local function spellIsFire(spell)
    for i=1, #spell.effects do
        local effect = spell.effects[i]
        if not fireEffects[effect.id] then
            return false
        end
    end
    return true
end

local function spellIsFrost(spell)
    for i=1, #spell.effects do
        local effect = spell.effects[i]
        if not frostEffects[effect.id] then
            return false
        end
    end
    return true
end


return {
    id = "aspectOfFlame",
    name = "Aspect de flamme",
    description = (
        "Vous avez une affinit naturelle pour le feu. Peut-tre tes-vous descendu d'un lmentaire de feu, " ..
        "mais il vaut mieux ne pas se demander comment cela s'est produit. Nanmoins, vous tes non seulement insensible au feu, " ..
        "vous pouvez galement lancer avec une trs grande aisance des sorts de feu. Cependant, " ..
        "vous tes vulnrable au froid et avez du mal  lancer des sorts de froid."
    ),
    doOnce = function()
        local immuneToFireSpell = tes3spell.create(
            "charBG_aspectOfFlame_immunity",
            "Aspect de flamme"
        )
        immuneToFireSpell.castType = tes3.spellType.ability
        immuneToFireSpell.effects[1] = {
            id = tes3.effect.resistFire,
            rangeType = tes3.effectRange.self,
            min = 100,
            max = 100,
        }
        mwscript.addSpell{ reference = tes3.player, spell = immuneToFireSpell }
    end,
    callback = function()
        local function spellCast(e)
            local data = getData()
            if data.currentBackground == "aspectOfFlame" then
                if e.caster == tes3.player then
                    if e.source and spellIsFire(e.source) then
                        local newCastChance = math.min(100, e.castChance * 2)
                        mwse.log("Aspect of Fire casting fire spell, changing cast chance from %s to %s",
                            e.castChance, newCastChance)
                        e.castChance = newCastChance
                    end
                end
            end
        end

        event.unregister("spellCast", spellCast)
        event.register("spellCast", spellCast)
    end
}
