local dremoraBloodDoOnce
local dremoraInterruptChance = 0.5
local dremoraDoAttack

local getData = function()
    local data = tes3.player.data.merBackgrounds or {}
    data.dremoraBlood = data.dremoraBlood or {
        dremoraKilled = 0
    }
    return data
end

return {
    id = "dremoraBlood",
    name = "Sang du drmora",
    description = (
        "Il y a longtemps, vous avez accompli un sombre rituel pour infuser votre sang avec celui d'un drmora. " ..
        "Bien que cela ait accru votre affinit pour la magie, cet acte l'a galement grandement courrouc. " ..
        "De temps  autres, le drmora se rendra sur Nirn et vous traquera. " ..
        "Chaque fois qu'il sera vaincu, vous absorberez son sang, ce qui augmente tous vos talents de Magie d'un point."
    ),
    callback = function()
        --calculate whether to replace interrupt creature with dremora
        local function calcRestInterrupt(e)
            local data = getData()
            if data.currentBackground == "dremoraBlood" then
                --One dremora killed every two levels, starting at lvl 2
                local readyForDremora = (
                    (tes3.player.object.level - 2) >= (  data.dremoraBlood.dremoraKilled * 2 )
                )
                if readyForDremora then
                    local rand = math.random()
                    if rand < dremoraInterruptChance then
                        dremoraDoAttack = true
                        e.count = 1
                        e.hour = math.random(1, 3)
                    end
                end
            end
        end
        --replace interrupt creature with dremora
        local function restInterrupt(e)
            local data = getData()
            if data.currentBackground == "dremoraBlood" then
                if dremoraDoAttack then
                    dremoraDoAttack = false
                    e.creature = tes3.getObject("mer_bg_dremList")
                    local pcName = tes3.player.object.name
                    local introPhrases = {
                        "Rends-moi mon sang, mortel !",
                        string.format("C'est la fin, %s !", pcName),
                        string.format("Ton me est  moi, %s !",  pcName),
                        "Tu regretteras le jour o tu as pris mon sang, mortel !",
                        string.format("Sois maudit, %s ! Je te tuerai la prochaine fois !", pcName)
                    }
                    tes3.playSound({
                        sound = "dremora scream"
                    })
                    tes3.messageBox( introPhrases[ math.random(#introPhrases)] )
                end
            end
        end

        --When dremora is dead, increase all magic skills by +1
        local function onDeath(e)
            local data = getData()
            if data.currentBackground == "dremoraBlood" then
                if string.find(e.reference.baseObject.id, "mer_bg_drem") then
                    local deathPhrases = {

                    }
                    tes3.messageBox( deathPhrases[math.random(#deathPhrases)] )

                    tes3.playSound({ sound = "dremora moan"})
                    mwscript.disable({ reference = e.mobile})

                    data.dremoraBlood.dremoraKilled = data.dremoraBlood.dremoraKilled + 1
                    local magicSkills = {
                        "illusion",
                        "alchemy",
                        "alteration",
                        "conjuration",
                        "destruction",
                        "enchant",
                        "mysticism",
                        "restoration"
                    }
                    for _, skill in ipairs(magicSkills) do
                        tes3.modStatistic({
                            reference = tes3.player,
                            skill = tes3.skill[skill],
                            value = 1
                        })
                    end
                    tes3.messageBox({
                        message = "Le sang du drmora coule dans vos veines. Vos talents magiques ont progress !",
                        buttons = { "OK" }
                    })
                end
            end
        end

        --Prevent looting dremora
        local function onActivate(e)
            if e.target and string.find(e.target.baseObject.id, "mer_bg_drem") then
                return false
            end
        end

        if dremoraBloodDoOnce then return end
        dremoraBloodDoOnce = true

        event.register("calcRestInterrupt", calcRestInterrupt)
        event.register("restInterrupt", restInterrupt)
        event.register("death", onDeath)
        event.register("activate", onActivate)
    end
}
