local escapedSlaveInterruptChance = 0.50
local currentSlaver

local defaultSlaverList = {
    { id = "mer_bg_headhunter_01", list = "mer_bg_headhunterList_01", hasFought = false},
    { id = "mer_bg_headhunter_02", list = "mer_bg_headhunterList_02", hasFought = false},
    { id = "mer_bg_headhunter_03", list = "mer_bg_headhunterList_03", hasFought = false},
    { id = "mer_bg_headhunter_04", list = "mer_bg_headhunterList_04", hasFought = false},
    { id = "mer_bg_headhunter_05", list = "mer_bg_headhunterList_05", hasFought = false},
    { id = "mer_bg_slavemaster", list = "mer_bg_slavemasterList", hasFought = false},
}

local function getData()
    local data = tes3.player.data.merBackgrounds or {}
    return data
end

local function getEscapedSlaveData()
    return getData().escapedSlave or {}
end

local function isActive()
    return getData().currentBackground == "escapedSlave"
end

local function isValidSlaver(slaver)
    if getData().testMode then
        return slaver.hasFought ~= true
    end
    return slaver.hasFought ~= true
        and tes3.getObject(slaver.id).level <= tes3.player.object.level
end


local function calcRestInterrupt(e)
    local data = getEscapedSlaveData()
    if isActive() then
        local rand = math.random()
        if rand < escapedSlaveInterruptChance then
            for _, slaver in ipairs(data.slavers) do
                if isValidSlaver(slaver) then
                    currentSlaver = slaver
                    e.count = 1
                    e.hour = math.random(1, 3)
                    break
                end
            end

        end
    end
end

local function restInterrupt(e)
    if isActive() then
        if currentSlaver and not currentSlaver.hasFought then
            currentSlaver.hasFought = true
            e.creature = tes3.getObject(currentSlaver.list)
        end
    end
end

return {
    id = "escapedSlave",
    name = "Esclave en fuite",
    description = (
        "Vous tes la proprit d'un riche esclavagiste, mais vous avez russi  vous chapper... du moins, pour le moment. " ..
        "Votre ancien propritaire ne tolre pas une telle dsobissance, et a envoy une quipe de chasseurs de ttes " ..
        "pour vous retrouver et vous tuer, de peur que les autres esclaves n'aient l'espoir d'tre libres un jour.\n\n" ..

        "Rserv aux Khajiits et aux Argoniens."
    ),
    checkDisabled = function()
        local race = tes3.player.object.race.id
        local isSlaveRace = ( race == "Argonian" or race == "Khajiit" )
        return not isSlaveRace
    end,
    doOnce = function(data)
        data.escapedSlave = data.escapedSlave or {
            slaversKilled = 0,
            slavers = defaultSlaverList
        }
        tes3.addItem{
            reference = tes3.player,
            item = "slave_bracer_left",
        }
        tes3.addItem{
            reference = tes3.player,
            item = "slave_bracer_right",
        }
        timer.delayOneFrame(function()
            tes3.mobilePlayer:equip{ item = "slave_bracer_left", playSound = false }
            tes3.mobilePlayer:equip{ item = "slave_bracer_right", playSound = false }
        end)
    end,
    callback = function()
        event.unregister("calcRestInterrupt", calcRestInterrupt)
        event.register("calcRestInterrupt", calcRestInterrupt)
        event.unregister("restInterrupt", restInterrupt)
        event.register("restInterrupt", restInterrupt)
    end
}
